/*
  TEST 1 for the amomii IR Shield. Written by Shaun Armstrong on July 7, 2024.
  Upload this code do you UNO with the IR Hacks shield attached. 
  After the upload is complete, open the serial monitor and follow the instructions.
  Note: you must set the Baud Rate 115200
*/

#include <Wire.h>  //i2c
#include <lcdgfx.h>
#include "lcdgfx_gui.h"
#include "DHT.h"
#include "images.h"


/////////////////////////////////// Functional Declarations ////////////////////////////

int longPressTime = 1000;

/////////////////////////////////// Blink //////////////////////////////
DisplaySSD1306_128x64_I2C display(-1);

// Define the pins for the buttons
#define SW1 A1
#define SW2 A2
#define SW3 A3

//////////////////////////////////// Keypad //////////////////////////

int rows[] = { 7, 6, 5, 4 };
int columns[] = { 11, 10, 9, 8 };
const int ROWS = 4;
const int COLS = 4;

char keys[ROWS][COLS] = {
  { '1', '2', '3', 'A' },
  { '4', '5', '6', 'B' },
  { '7', '8', '9', 'C' },
  { '*', '0', '#', 'D' }
};


///////////////////// BUZZER /////////////////////////

int buz = 12;

///////////////////// DHT11 /////////////////////////

#define DHTPIN A0
#define DHTTYPE DHT11
DHT dht(DHTPIN, DHTTYPE);

void setup() {

  Serial.begin(115200);

  //////////////////////////////////// blink ////////////////////
  display.begin();
  display.setFixedFont(ssd1306xled_font6x8);


  display.getInterface().flipHorizontal(1);
  display.getInterface().flipVertical(1);

  display.clear();

  display.fill(0x00);
  display.drawBitmap1(0, 0, 128, 64, ir_logo);


  // Button setup
  pinMode(SW1, INPUT_PULLUP);
  pinMode(SW2, INPUT_PULLUP);
  pinMode(SW3, INPUT_PULLUP);

  //////////////////////////////////// Keyboard ///////////////////

  for (int i = 0; i < COLS; i++) {
    pinMode(columns[i], INPUT_PULLUP);
  }
  for (int i = 0; i < ROWS; i++) {
    pinMode(rows[i], OUTPUT);
    digitalWrite(rows[i], HIGH);
  }

  ////////////////////////////// Buzzer ////////////////////
  pinMode(buz, OUTPUT);

  ////////////////////////////// DHT11 ////////////////////
  dht.begin();
}

char checkKeypad() {
  bool flag = true;
  char KEY = 0;
  bool previousPress = false;
  while (flag == true) {
    // master 'for loop' with the variable 'i'
    bool buttonPress = false;
    for (int i = 0; i < ROWS; i++) {

      // write all rows HIGH
      for (int x = 0; x < ROWS; x++) {
        digitalWrite(rows[x], HIGH);
      }

      // set ONE row LOW
      digitalWrite(rows[i], LOW);

      for (int j = 0; j < COLS; j++) {
        // read each column
        int buttonState = digitalRead(columns[j]);
        if (buttonState == 0) {
          buttonPress = true;
          previousPress = true;
          KEY = keys[i][j];
        }
      }
    }
    if (previousPress == true && buttonPress == false) {
      flag = false;
      tone(buz, 880, 10);
    }
  }
  return (KEY);
}


void testKeys() {
  Serial.println(F("Let the test begin!"));

  // Array of keys to be tested
  char keysToTest[] = { '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '*', '#', 'A', 'B', 'C', 'D' };

  // Loop through each key in the array and prompt the user to press it
  for (int i = 0; i < sizeof(keysToTest); i++) {
    Serial.print(F("Press Button "));
    Serial.println(keysToTest[i]);

    // Wait until the correct key is pressed
    while (checkKeypad() != keysToTest[i]) {}

    Serial.print(F("Button "));
    Serial.print(keysToTest[i]);
    Serial.println(F(" Pressed"));
  }

  Serial.println(F("KEYPAD TEST COMPLETE!!!"));
  Serial.println();
  Serial.println(F("Playing a tune to test the buzzer..."));

  // Play a little tune
  playTune();

  Serial.println(F("If you heard beeps and boops during the keypad test, consider the buzzer tested!"));
}

void playTune() {
  int melody[] = {
    262, 294, 330, 349, 392, 440, 494, 523
  };

  int noteDurations[] = {
    4, 4, 4, 4, 4, 4, 4, 4
  };

  for (int thisNote = 0; thisNote < 8; thisNote++) {
    int noteDuration = 100 / noteDurations[thisNote];
    tone(buz, melody[thisNote], noteDuration);
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    noTone(buz);
  }
  Serial.println();
}

void testDHT11() {
  Serial.println(F("Testing DHT11 Sensor..."));
  delay(2000);  // Give sensor some time to get readings

  float h = dht.readHumidity();
  float t = dht.readTemperature();

  if (isnan(h) || isnan(t)) {
    Serial.println(F("Failed to read from DHT sensor!"));
    return;
  }

  Serial.print(F("Current Humidity: "));
  Serial.print(h);
  Serial.println(F("%"));

  Serial.print(F("Current Temperature: "));
  Serial.print(t);
  Serial.println(F("°C"));

  delay(1000);

  Serial.println(F("Does the reading feel about right? (Press Button 1 for yes, 2 for no)"));

  while (true) {

    char response = checkKeypad();
    if (response == '1') {
      Serial.println(F("DHT11 Sensor Test Passed!"));
      playTune();
      break;
    } else if (response == '2') {
      Serial.println(F("DHT11 Sensor Test Failed! Check the sensor connections and try again."));
      break;
    }
  }
}

void testBlink() {
  display.clear();
  Serial.println(F("Testing OLED and buttons (amomii Blink)..."));

  // Display test message on OLED
  Serial.println(F("Press SW1 on the amomii Blink"));

  while (true) {
    if (!digitalRead(SW1)) {
      tone(buz, 880, 10);
      display.printFixed(0, 0, "IT");

      break;
    }
  }
  Serial.println(F("Press SW2 on the amomii Blink"));

  while (true) {
    if (!digitalRead(SW2)) {
      tone(buz, 880, 10);
      display.printFixed(0, 20, "IS");

      break;
    }
  }

  Serial.println(F("Press SW3 on the amomii Blink"));

  while (true) {
    if (!digitalRead(SW3)) {
      tone(buz, 880, 10);
      display.printFixed(0, 35, "WORKING");
      break;
    }
  }

  Serial.println(F("Look at the Blink screen. Is it working?"));

  Serial.println(F("Press Button 1 for yes, 2 for no"));

  while (true) {

    char response = checkKeypad();
    if (response == '1') {
      Serial.println(F("Blink Test Passed!"));
      playTune();
      tone(buz, 880, 10);
      break;
    } else if (response == '2') {
      Serial.println(F("Blink Test Failed! Check the connections and try again."));
      tone(buz, 880, 10);
      display.clear();
      display.printFixed(0, 0, "LIAR!!!!!");
      break;
    }
  }
}


void failTune() {
  int melody[] = { 330, 294, 262 };   // Notes for the fail tune
  int noteDurations[] = { 4, 4, 4 };  // Duration of each note

  for (int thisNote = 0; thisNote < 3; thisNote++) {
    int noteDuration = 1000 / noteDurations[thisNote];
    tone(buz, melody[thisNote], noteDuration);
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    noTone(buz);
  }
}



void loop() {
  testKeys();
  delay(1000);
  Serial.println();
  testDHT11();
  delay(1000);
  Serial.println();
  display.fill(0x00);
  testBlink();
  display.fill(0x00);
  display.drawBitmap1(0, 0, 128, 64, ir_logo);
  delay(1000);
  Serial.println();

  Serial.println(F("TEST 1 COMPLETE"));
  Serial.println(F("Address any of the failed tests and try again!"));
  Serial.println();
  Serial.println(F("When eveything is working, move on to TEST 2"));
  while (true)
    ;
}
